@@ -82,8 +82,16 @@ const Key = new Lang.Class({
                                       style_class: 'keyboard-key' });
 
         button.key_width = this._key.width;
-        button.connect('button-press-event', Lang.bind(this, function () { this._key.press(); }));
-        button.connect('button-release-event', Lang.bind(this, function () { this._key.release(); }));
+        button.connect('button-press-event', Lang.bind(this,
+            function () {
+                this._key.press();
+                return Clutter.EVENT_PROPAGATE;
+            }));
+        button.connect('button-release-event', Lang.bind(this,
+            function () {
+                this._key.release();
+                return Clutter.EVENT_PROPAGATE;
+            }));
 
         return button;
     },
@@ -106,8 +114,16 @@ const Key = new Lang.Class({
             let label = this._getUnichar(extended_key);
             let key = new St.Button({ label: label, style_class: 'keyboard-key' });
             key.extended_key = extended_key;
-            key.connect('button-press-event', Lang.bind(this, function () { extended_key.press(); }));
-            key.connect('button-release-event', Lang.bind(this, function () { extended_key.release(); }));
+            key.connect('button-press-event', Lang.bind(this,
+                function () {
+                    extended_key.press();
+                    return Clutter.EVENT_PROPAGATE;
+                }));
+            key.connect('button-release-event', Lang.bind(this,
+                function () {
+                    extended_key.release();
+                    return Clutter.EVENT_PROPAGATE;
+                }));
             this._extended_keyboard.add(key);
         }
         this._boxPointer.bin.add_actor(this._extended_keyboard);
@@ -252,7 +268,10 @@ const Keyboard = new Lang.Class({
 
         if (!this._showIdleId)
             this._showIdleId = GLib.idle_add(GLib.PRIORITY_DEFAULT_IDLE,
-                                             Lang.bind(this, function() { this.Show(time); }));
+                                             Lang.bind(this, function() {
+                                                 this.Show(time);
+                                                 return GLib.SOURCE_REMOVE;
+                                             }));
     },
 
     _createLayersForGroup: function (gname) {
@@ -294,7 +313,7 @@ const Keyboard = new Lang.Class({
         else if (release && this._capturedPress)
             this._hideSubkeys();
 
-        return true;
+        return Clutter.EVENT_STOP;
     },
 
     _addRows : function (keys, layout) {
@@ -438,7 +457,6 @@ const Keyboard = new Lang.Class({
     _createSource: function () {
         if (this._source == null) {
             this._source = new KeyboardSource(this);
-            this._source.setTransient(true);
             Main.messageTray.add(this._source);
         }
     },
@@ -480,6 +498,7 @@ const Keyboard = new Lang.Class({
                                                    Lang.bind(this, function() {
                                                        this._clearKeyboardRestTimer();
                                                        this._show(monitor);
+                                                       return GLib.SOURCE_REMOVE;
                                                    }));
     },
 
@@ -505,6 +524,7 @@ const Keyboard = new Lang.Class({
                                                    Lang.bind(this, function() {
                                                        this._clearKeyboardRestTimer();
                                                        this._hide();
+                                                       return GLib.SOURCE_REMOVE;
                                                    }));
     },
 
