@@ -293,8 +293,13 @@ const NMConnectionSection = new Lang.Class({
     },
 
     removeConnection: function(connection) {
-        this._connectionItems.get(connection.get_uuid()).destroy();
-        this._connectionItems.delete(connection.get_uuid());
+        let uuid = connection.get_uuid();
+        let item = this._connectionItems.get(uuid);
+        if (item == undefined)
+            return;
+
+        item.destroy();
+        this._connectionItems.delete(uuid);
 
         let pos = this._connections.indexOf(connection);
         this._connections.splice(pos, 1);
@@ -1165,6 +1170,10 @@ const NMDeviceWireless = new Lang.Class({
             this._client.disconnect(this._wirelessHwEnabledChangedId);
             this._wirelessHwEnabledChangedId = 0;
         }
+        if (this._dialog) {
+            this._dialog.destroy();
+            this._dialog = null;
+        }
 
         this.item.destroy();
     },
@@ -1857,8 +1866,7 @@ const NMApplet = new Lang.Class({
 
     _updateIcon: function() {
         if (!this._client.networking_enabled || !this._mainConnection) {
-            this._primaryIndicator.icon_name = 'network-offline-symbolic';
-            this._primaryIndicator.visible = true;
+            this._primaryIndicator.visible = false;
         } else {
             let dev = this._mainConnection._primaryDevice;
             this._primaryIndicator.visible = (dev != null);
