@@ -812,9 +812,9 @@ const ThumbnailsBox = new Lang.Class({
                     return false;
 
                 if (isWindow)
-                    return window.get_workspace() >= newWorkspaceIndex && winActor != source;
+                    return window.get_workspace().index() >= newWorkspaceIndex && winActor != source;
                 else
-                    return window.get_workspace() >= newWorkspaceIndex;
+                    return window.get_workspace().index() >= newWorkspaceIndex;
             });
 
             this._spliceIndex = newWorkspaceIndex;
@@ -822,7 +822,7 @@ const ThumbnailsBox = new Lang.Class({
             // ... move them down one.
             windows.forEach(function(winActor) {
                 let window = winActor.meta_window;
-                window.change_workspace_by_index(window.get_workspace() + 1, true);
+                window.change_workspace_by_index(window.get_workspace().index() + 1, true);
             });
 
             if (isWindow)
